@@ -41,3 +41,8 @@ void add_signal(int sig, void (*handler)(), int exit_after)
 	else
 		signal(sig, call_handlers);
 }
+
+void reset_signal(int sig)
+{
+	signal(sig, call_handlers);
+}
