@@ -11,6 +11,10 @@ proc InitGlobal {} {
 
     global Stat
 
+    # input/output pipe
+    set Stat(input) stdin
+    set Stat(output) stdout
+
     # time table
     set Stat(MaxSecs) 0
     set Stat(LastSec) 0
@@ -24,6 +28,7 @@ proc InitGlobal {} {
     set Stat(Playing) 0
     set Stat(Paused) 0
     set Stat(Blinking) 0
+    set Stat(Buttons) -1
 
     # MIDI file list
     set Stat(CurIdx) -1
@@ -35,22 +40,22 @@ proc InitGlobal {} {
 
     global Config
     # playing mode
-    set Config(Tracing) 0
     set Config(RepeatPlay) 0
     set Config(ShufflePlay) 0
     set Config(AutoStart) 0
     set Config(AutoExit) 0
     set Config(ConfirmExit) 1
+    set Stat(Tracing) 1
 
     # display configuration
     set Config(Disp:file) 1
-    set Config(Disp:time) 1
+    set Config(Disp:time) 0
     set Config(Disp:text) 0
-    set Config(Disp:volume) 1
+    set Config(Disp:volume) 0
     set Config(Disp:button) 1
-    set Config(Disp:trace) 0
+    set Config(Disp:trace) 1
 
-    set Config(Mode:chorus) 4
+    set Config(Mode:chorus) 2
     set Config(Mode:reverb) 4
 
     set Config(CurFileMode) 0
@@ -60,6 +65,31 @@ proc InitGlobal {} {
     set Config(CurBass) 5
     set Config(CurTreble) 9
 
+    set Stat(ChorusList) {
+	{0 "Chorus 1"}
+	{1 "Chorus 2"}
+	{2 "Chorus 3"}
+	{3 "Chorus 4"}
+	{4 "Feedback"}
+	{5 "Flanger"}
+	{6 "Short Delay"}
+	{7 "Short Delay 2"}
+    }
+
+    set Stat(ReverbList) {
+	{0 "Room 1"}
+	{1 "Room 2"}
+	{2 "Room 3"}
+	{3 "Hall 1"}
+	{4 "Hall 2"}
+	{5 "Plate"}
+	{6 "Delay"}
+	{7 "Panning Delay"}
+    }
+
+    set Config(UserChorus) {}
+    set Config(UserReverb) {}
+
     wm title . "TkAWEMidi"
     wm iconname . "TkAWEMidi"
     #wm iconbitmap . @$bitmap_path/tkawe.xbm
@@ -72,12 +102,13 @@ proc InitGlobal {} {
 proc HandleInput {} {
     global Stat Config
 
-    set mlist [gets stdin]
-    set msg [lindex $mlist 0]
+    set mlist [gets $Stat(input)]
+    set msg [string range $mlist 0 3]
+    set args [string range $mlist 5 end]
 
     if {$msg == "TIME"} {
 	# set total time
-	set csecs [expr [lindex $mlist 1] / 100]
+	set csecs [expr $args / 100]
 	set Stat(TotalTimeStr) [sec2time $csecs]
 	set Stat(MaxSecs) $csecs
 	set tics [expr $csecs / 8]
@@ -85,13 +116,13 @@ proc HandleInput {} {
 	.body.time.scale configure -tickinterval $tics -to $csecs
 	SetTime 0
 
-    } elseif {$msg == "MVOL"} {
+    } elseif {$msg == "VOLM"} {
 	# set master volume
-	SetVolume [lindex $mlist 1]
+	SetVolume $args
 
     } elseif {$msg == "FILE"} {
 	# set playing file
-	set Stat(CurFile) [join [lrange $mlist 1 end]]
+	set Stat(CurFile) $args
 	wm title . "TkAWEMidi: $Stat(CurFile)"
 	wm iconname . "TkAWEMidi: $Stat(CurFile)"
 	.body.curfile.title configure -text $Stat(CurFile)
@@ -102,10 +133,16 @@ proc HandleInput {} {
 	}
 	AppendMsg "------"
 
+    } elseif {$msg == "TITL"} {
+	# set title name
+	set title $args
+	catch {set title [kanji conversion [kanji code $title] EUC $title]}
+	.body.curfile.title configure -text $title
+
     } elseif {$msg == "LIST"} {
 	# set file list
 	.body.file.list delete 0 end
-	set Stat(MaxFiles) [lindex $mlist 1]
+	set Stat(MaxFiles) $args
 	set Stat(FileList) {}
 	for {set i 0} {$i < $Stat(MaxFiles)} {incr i} {
 	    set file [gets stdin]
@@ -123,7 +160,7 @@ proc HandleInput {} {
 	if {$Stat(CurIdx) < 0} {set Stat(CurIdx) 0}
 	SelectNumber 
 
-    } elseif {$msg == "NEXT" || $msg == "TEND"} {
+    } elseif {$msg == "NEXT"} {
 	# next file
 	incr Stat(CurIdx)
 	if {$Stat(CurIdx) >= $Stat(MaxFiles)} {
@@ -139,7 +176,6 @@ proc HandleInput {} {
 
     } elseif {$msg == "CMSG"} {
 	# put message
-	set type [lindex $mlist 1]
 	set str [gets stdin]
 	AppendMsg $str
 
@@ -147,18 +183,10 @@ proc HandleInput {} {
 	error [format "%s: %s" $Stat(NextFile) [gets stdin]]
 	WriteMsg "NEXT"
 
-    } elseif {$msg == "QUIT"} {
-	# quit
-	exit
-    } elseif {$msg == "RSTA"} {
-	# restart file
-	SelectNumber
     } elseif {$msg == "CHRS"} {
-	set Config(Mode:chorus) [lindex $mlist 1]
-	SetChorus
+	set Config(Mode:chorus) $args
     } elseif {$msg == "EVRB"} {
-	set Config(Mode:reverb) [lindex $mlist 1]
-	SetReverb
+	set Config(Mode:reverb) $args
     }
 }
 
@@ -175,7 +203,7 @@ proc MakeShuffleList {} {
     set Stat(ShuffleList) {}
     set len $Stat(MaxFiles)
     while {$len > 0} {
-	set pos [my-random $len]
+	set pos [random $len]
 	lappend Stat(ShuffleList) [lindex $tmplist $pos]
 	set tmplist [lreplace $tmplist $pos $pos]
 	set len [expr $len - 1]
@@ -231,12 +259,16 @@ proc SelectNumber {} {
 
     if {$idx >= 0} {
 	.body.file.list select set $idx
+	.body.file.list see $idx
 	LoadCmd $idx
 	set Stat(Playing) 1
+	.body.curfile.index configure -text [format "%03d/%03d"\
+		[expr $idx + 1] $Stat(MaxFiles)]
     } else {
 	SetTime 0
 	.body.curfile.title configure -text "--------"
 	.body.curfile.time configure -text "00:00"
+	.body.curfile.index configure -text "000/000"
 	set Stat(Playing) 0
 	set Stat(Paused) 0
     }
@@ -264,14 +296,14 @@ proc SetTime {val} {
     .body.time.label configure\
 	    -text "$curt / $Stat(TotalTimeStr)"
     .body.time.scale set $val
-    DispButtonPlay
+    ToggleButtonLump
 }
 
 
 #
 # colorize buttons with each state
 #
-proc DispButtonPlay {} {
+proc ToggleButtonLump {} {
     global Stat
     if {$Stat(Playing)} {
 	if {$Stat(Blinking)} {
@@ -282,15 +314,75 @@ proc DispButtonPlay {} {
 	    set Stat(Blinking) 1
 	}
 	.body.button.play configure -fg $color -activeforeground $color
-    } else {
-	.body.button.play configure -fg black -activeforeground black
     }
+}
+    
+#
+# enable/disable button & menu status
+#
+proc DispButtonPlay {} {
+    global Stat
 
+    ToggleButtonLump
     if {$Stat(Playing) && $Stat(Paused)} {
 	.body.button.pause configure -fg red -activeforeground red
     } else {
 	.body.button.pause configure -fg black -activeforeground black
     }
+
+    if {$Stat(Playing) && $Stat(Paused)} {
+	if {$Stat(Buttons) != 2} {
+	    .body.time.scale configure -state disabled
+	    .body.button.play configure -state disabled
+	    .body.button.next configure -state disabled
+	    .body.button.prev configure -state disabled
+	    .body.button.pause configure -state normal
+	    .body.button.fwrd configure -state disabled
+	    .body.button.back configure -state disabled
+	    set Stat(Buttons) 2
+	}
+    } elseif {$Stat(Playing)} {
+	if {$Stat(Buttons) != 1} {
+	    .body.time.scale configure -state normal
+	    .body.button.play configure -state normal
+	    .body.button.next configure -state normal
+	    .body.button.prev configure -state normal
+	    .body.button.pause configure -state normal
+	    .body.button.fwrd configure -state normal
+	    .body.button.back configure -state normal
+	    .body.button.stop configure -state normal
+	    set Stat(Buttons) 1
+	}
+    } else {
+	if {$Stat(Buttons) != 0} {
+	    .body.time.scale configure -state disabled
+	    .body.button.play configure -fg black -activeforeground black
+	    .body.button.play configure -state normal
+	    .body.button.next configure -state disabled
+	    .body.button.prev configure -state disabled
+	    .body.button.pause configure -state disabled
+	    .body.button.fwrd configure -state disabled
+	    .body.button.back configure -state disabled
+	    .body.button.stop configure -state disabled
+	    set Stat(Buttons) 0
+	}
+    }
+
+    if {$Stat(Playing)} {
+	set w .menu.file.m
+	$w entryconfigure 1 -state disabled
+	$w entryconfigure 2 -state disabled
+	#$w entryconfigure 3 -state disabled
+	set w .menu.mode.m
+	$w entryconfigure 2 -state disabled
+    } else {
+	set w .menu.file.m
+	$w entryconfigure 1 -state normal
+	$w entryconfigure 2 -state normal
+	#$w entryconfigure 3 -state normal
+	set w .menu.mode.m
+	$w entryconfigure 2 -state normal
+    }
 }    
 
 #
@@ -306,14 +398,15 @@ proc SetVolume {val} {
 
 #----------------
 # write message
-# messages are: PREV, NEXT, ZAPP, QUIT, FWRD, BACK, RSET, STOP
+# messages are: PREV, NEXT, ZAPP, QUIT, FWRD, BACK, RSTA, STOP
 #	LOAD\n<filename>, JUMP <time>
 #	VOLM <%>, VBAS <val>, VTRB <val>
 #
 
 proc WriteMsg {str} {
-    puts stdout $str
-    flush stdout
+    global Stat
+    puts $Stat(output) $str
+    flush $Stat(output)
 }
 
 
@@ -364,7 +457,7 @@ proc LoadCmd {idx} {
     AppendMsg ""
     VolumeCmd $Config(CurVol) 1
     BassCmd $Config(CurBass)
-    TrebleCmd $Config(CurBass)
+    TrebleCmd $Config(CurTreble)
 }
 
 #
@@ -401,9 +494,10 @@ proc StopCmd {} {
     global Stat Config
     if {$Stat(Playing)} {
 	WriteMsg "QUIT"
+	SetTime 0
 	set Stat(CurIdx) -1
 	SelectNumber
-	if {$Config(Tracing)} {
+	if {$Stat(Tracing)} {
 	    TraceReset
 	}
 	wm title . "TkAWEMidi"
@@ -417,13 +511,13 @@ proc StopCmd {} {
 proc QuitCmd {} {
     global Config Stat
     if {$Config(AutoExit) || !$Config(ConfirmExit)} {
-	WriteMsg "ZAPP"
+	ReallyQuit
 	return
     }
     set oldpause $Stat(Paused)
     if {!$oldpause} {PauseCmd}
     if {[question "Really Quit TkAWEMidi?" 0]} {
-	WriteMsg "ZAPP"
+	ReallyQuit
 	return
     }
     if {!$oldpause} {PauseCmd}
@@ -449,6 +543,17 @@ proc NextCmd {} {
     }
 }
 
+#
+# restart play
+#
+proc RestartCmd {} {
+    global Stat
+    if {$Stat(Playing)} {
+	WriteMsg "RSTA"
+    }
+}
+
+
 #
 # forward/backward 2 secs
 #
@@ -492,6 +597,15 @@ proc TrebleCmd {val} {
     WriteMsg [format "VTRB %d" $val]
 }
 
+proc BaseUpCmd {} {
+    WriteMsg [format "OFST 1"]
+}
+
+proc BaseDownCmd {} {
+    WriteMsg [format "OFST -1"]
+}
+
+
 #----------------
 # display configured tables
 #
@@ -516,7 +630,11 @@ proc SaveConfig {} {
     if {$fd != ""} {
 	puts $fd "global Config"
 	foreach i [array names Config] {
-	    puts $fd "set Config($i) $Config($i)"
+	    if {[llength $Config($i)] > 1} {
+		puts $fd "set Config($i) {$Config($i)}"
+	    } else {
+		puts $fd "set Config($i) $Config($i)"
+	    }
 	}
 	close $fd
     }
@@ -533,17 +651,19 @@ proc LoadConfig {} {
 #
 # from command line
 #
-proc InitCmdLine {argc argv} {
-    global Config
-    set Config(Disp:trace) 0
+proc ParseCmdLine {} {
+    global Config Stat argc argv
+    set Stat(FileList) {}
+    set Stat(MaxFiles) 0
     for {set i 0} {$i < $argc} {incr i} {
-	if {[lindex $argv $i] == "-mode"} {
+	set v [lindex $argv $i]
+	if {$v == "-effect"} {
+	    incr i
+	    SetUserEffects [lindex $argv $i]
+	} elseif {$v == "-mode"} {
 	    incr i
 	    set mode [lindex $argv $i]
-	    if {$mode == "trace"} {
-		set Config(Tracing) 1
-		set Config(Disp:trace) 1
-	    } elseif {$mode == "shuffle"} {
+	    if {$mode == "shuffle"} {
 		set Config(ShufflePlay) 1
 	    } elseif {$mode == "normal"} {
 		set Config(ShufflePlay) 0
@@ -554,10 +674,63 @@ proc InitCmdLine {argc argv} {
 	    } elseif {$mode == "repeat"} {
 		set Config(RepeatPlay) 1
 	    }
+	} elseif {![regexp "^-" $v dummy]} {
+	    if {[regexp "^\(.*\)/" $v dummy root]} {
+		set v $root
+	    }
+	    if {[file isdirectory $v]} {
+		if {$v == "."} {
+		    set pattern "*.mid*"
+		} else {
+		    set pattern "$v/*.mid*"
+		}
+		foreach f [lsort [glob -nocomplain $pattern]] {
+		    if {[file isfile $f]} {
+			lappend Stat(FileList) $f
+			incr Stat(MaxFiles)
+		    }
+		}
+	    } elseif {[file isfile $v]} {
+		lappend Stat(FileList) $v
+		incr Stat(MaxFiles)
+	    }
 	}
     }
 }
 
+#
+# read a config file for user defined chorus/reverb modes
+#
+proc SetUserEffects {file} {
+    global Stat Config
+    set fp [open $file r]
+    if {$fp == ""} {return}
+    set Config(UserChorus) {}
+    set Config(UserReverb) {}
+    gets $fp line
+    while {![eof $fp]} {
+	if {[regexp "^\(\[crCR\]\)\[a-zA-Z\]*:\(\[0-9\]+\):\(\[^:\]+\):"\
+		$line dummy type mode name]} {
+	    if {$type == "c" || $type == "C"} {
+		lappend Config(UserChorus) [list $mode $name]
+	    } else {
+		lappend Config(UserReverb) [list $mode $name]
+	    }
+	}
+	if {[regexp "\\$" $line]} {
+	    while {1} {
+		gets $fp line
+		if {[eof $fp]} {close $fp; return}
+		if {![regexp "\\$" $line]} {break}
+	    }
+	} else {
+	    gets $fp line
+	}
+    }
+
+    close $fp
+}
+
 
 #
 # selection callback of the playing file from listbox
@@ -574,6 +747,9 @@ proc SelectList {lw pos} {
 	} else {
 	    set Stat(CurIdx) $idx
 	}
+	if {$Stat(Playing) && $Stat(Paused)} {
+	    PauseCmd
+	}
 	set Stat(Playing) 1
 	SelectNumber
     }
@@ -581,26 +757,34 @@ proc SelectList {lw pos} {
     
 
 #
-#
+# open file browser and append some files
 #
 proc OpenFiles {} {
     global Stat
+    if {$Stat(Playing)} {
+	return
+    }
     set files [filebrowser .browser "" "*.mid*"]
     if {$files != ""} {
-	set Stat(MaxFiles) [expr $Stat(MaxFiles) + [llength $files]]
 	foreach i $files {
-	    .body.file.list insert end $i
-	    lappend Stat(FileList) $i
+	    if {[lsearch -exact $Stat(FileList) $i] == -1} {
+		.body.file.list insert end $i
+		lappend Stat(FileList) $i
+		incr Stat(MaxFiles)
+	    }
 	}
-	# MakeShuffleList
+	MakeShuffleList
     }
 }
 
 #
-#
+# clear file list
 #
 proc CloseFiles {} {
     global Stat
+    if {$Stat(Playing) || $Stat(MaxFiles) == 0} {
+	return
+    }
     if {[question "Really Clear List?" 0]} {
 	StopCmd
 	.body.file.list delete 0 end
@@ -610,7 +794,10 @@ proc CloseFiles {} {
     }
 }
 
-proc ToggleCurFileMode {} {
+#
+# toggle time display mode
+#
+proc ToggleTimeDispMode {} {
     global Config Stat
     if {$Config(CurFileMode) == 0} {
 	set Config(CurFileMode) 1
@@ -620,6 +807,33 @@ proc ToggleCurFileMode {} {
     SetTime $Stat(LastSec)
 }
 
+#
+# select and edit options for each midi file
+#
+proc EditIndexInList {} {
+    global Config Stat
+    set top .selediti
+    set w [my-dialog $top "Midi File Selection" 0 0 [list\
+	    [list "Close" "destroy $top"]\
+	    [list "Edit Default" "EditIndexFile .editor {}"]]]
+    set lw [make-listbox $w 36 10 0 single]
+    foreach i $Stat(FileList) {
+	$lw insert end $i
+    }
+    bind $lw <ButtonRelease-1> {DoEditIndex %W %y}
+
+    tkwait visibility $top
+    tkwait window $top
+}
+
+proc DoEditIndex {w y} {
+    global Config Stat
+    set idx [$w nearest $y]
+    if {$idx >= 0 && $idx < $Stat(MaxFiles)} {
+	EditIndexFile .editor [lindex $Stat(FileList) $idx]
+    }
+}
+
 #----------------
 # create main window
 #
@@ -639,6 +853,8 @@ proc CreateWindow {} {
 	    -command OpenFiles
     .menu.file.m add command -label "Clear List" -underline 0\
 	    -command CloseFiles
+    .menu.file.m add command -label "Edit Options" -underline 0\
+	    -command EditIndexInList
     .menu.file.m add command -label "Save Config" -underline 0\
 	    -command SaveConfig
     .menu.file.m add command -label "About" -underline 0\
@@ -679,51 +895,41 @@ proc CreateWindow {} {
 	    -variable Config(Disp:volume) -command "DispTables"
     .menu.disp.m add check -label "Buttons" -underline 0\
 	    -variable Config(Disp:button) -command "DispTables"
-    if {$Config(Tracing)} {
+    if {$Stat(Tracing)} {
 	.menu.disp.m add check -label "Trace" -underline 1\
 		-variable Config(Disp:trace) -command "DispTables"
     }
 
     # chorus and reverb mode menu
-    menubutton .menu.rc -text "Reverb/Chorus" -menu .menu.rc.m\
+    menubutton .menu.c -text "Chorus" -menu .menu.c.m\
 	    -underline 0
-    menu .menu.rc.m
-    .menu.rc.m add radio -label "Chorus 1"\
-	    -variable Config(Mode:chorus) -value 0 -command "SetChorus"
-    .menu.rc.m add radio -label "Chorus 2"\
-	    -variable Config(Mode:chorus) -value 1 -command "SetChorus"
-    .menu.rc.m add radio -label "Chorus 3"\
-	    -variable Config(Mode:chorus) -value 2 -command "SetChorus"
-    .menu.rc.m add radio -label "Chorus 4"\
-	    -variable Config(Mode:chorus) -value 3 -command "SetChorus"
-    .menu.rc.m add radio -label "Feedback"\
-	    -variable Config(Mode:chorus) -value 4 -command "SetChorus"
-    .menu.rc.m add radio -label "Flanger"\
-	    -variable Config(Mode:chorus) -value 5 -command "SetChorus"
-    .menu.rc.m add radio -label "Short Delay"\
-	    -variable Config(Mode:chorus) -value 6 -command "SetChorus"
-    .menu.rc.m add radio -label "Short Delay 2"\
-	    -variable Config(Mode:chorus) -value 7 -command "SetChorus"
-    .menu.rc.m add separator
+    menu .menu.c.m
+    foreach i $Stat(ChorusList) {
+	.menu.c.m add radio -label [lindex $i 1]\
+		-variable Config(Mode:chorus) -value [lindex $i 0]\
+		-command "SetChorus"
+    }
+    foreach i $Config(UserChorus) {
+	.menu.c.m add radio -label [lindex $i 1]\
+		-variable Config(Mode:chorus) -value [lindex $i 0]\
+		-command "SetChorus"
+    }
 
-    .menu.rc.m add radio -label "Room 1"\
-	    -variable Config(Mode:reverb) -value 0 -command "SetReverb"
-    .menu.rc.m add radio -label "Room 2"\
-	    -variable Config(Mode:reverb) -value 1 -command "SetReverb"
-    .menu.rc.m add radio -label "Room 3"\
-	    -variable Config(Mode:reverb) -value 2 -command "SetReverb"
-    .menu.rc.m add radio -label "Hall 1"\
-	    -variable Config(Mode:reverb) -value 3 -command "SetReverb"
-    .menu.rc.m add radio -label "Hall 2"\
-	    -variable Config(Mode:reverb) -value 4 -command "SetReverb"
-    .menu.rc.m add radio -label "Plate"\
-	    -variable Config(Mode:reverb) -value 5 -command "SetReverb"
-    .menu.rc.m add radio -label "Delay"\
-	    -variable Config(Mode:reverb) -value 6 -command "SetReverb"
-    .menu.rc.m add radio -label "Panning Delay"\
-	    -variable Config(Mode:reverb) -value 7 -command "SetReverb"
+    menubutton .menu.r -text "Reverb" -menu .menu.r.m\
+	    -underline 0
+    menu .menu.r.m
+    foreach i $Stat(ReverbList) {
+	.menu.r.m add radio -label [lindex $i 1]\
+		-variable Config(Mode:reverb) -value [lindex $i 0]\
+		-command "SetReverb"
+    }
+    foreach i $Config(UserReverb) {
+	.menu.r.m add radio -label [lindex $i 1]\
+		-variable Config(Mode:reverb) -value [lindex $i 0]\
+		-command "SetReverb"
+    }
 
-    pack .menu.file .menu.mode .menu.disp .menu.rc -side left
+    pack .menu.file .menu.mode .menu.disp .menu.c .menu.r -side left
 
     # display body
     frame .body -relief flat
@@ -731,23 +937,22 @@ proc CreateWindow {} {
 
     # current playing file
     frame .body.curfile -relief flat
-    label .body.curfile.title -text "--------" -relief flat -width 25
-    button .body.curfile.time -text "00:00" -relief ridge\
-	    -command "ToggleCurFileMode"
-    pack .body.curfile.title -side left -expand 1
+    label .body.curfile.index -text "000/000" -relief flat -anchor w
+    label .body.curfile.title -text "--------" -relief flat -width 23\
+	    -anchor w
+    button .body.curfile.time -text "00:00" -relief ridge -width 5\
+	    -command "ToggleTimeDispMode"
+    pack .body.curfile.index .body.curfile.title -side left -expand 1
     pack .body.curfile.time -side right
     pack .body.curfile -side top -fill x -expand 1
 
     # playing files list
     frame .body.file -relief raised -bd 1
-    scrollbar .body.file.bar -relief sunken\
-	    -command ".body.file.list yview"
-    pack .body.file.bar -side right -fill y
-    listbox .body.file.list -width 36 -height 10 -relief sunken -bd 2\
-	    -yscroll ".body.file.bar set"
+    make-listbox .body.file 36 10 0 single
     bind .body.file.list <Button-1> {SelectList %W %y}
-
-    pack .body.file.list -side top -expand 1 -fill both
+    foreach i $Stat(FileList) {
+	.body.file.list insert end $i
+    }
 
     # time label and scale
     frame .body.time -relief raised -bd 1
@@ -778,9 +983,8 @@ proc CreateWindow {} {
     #pack .body.volume.label -side top
     scale .body.volume.scale -orient horizontal -length 280\
 	    -from 0 -to 200 -tickinterval 25\
-	    -showvalue true -label "Volume"\
-	    -command VolumeCmd
-    #bind .body.volume.scale <ButtonRelease-1> {VolumeCmd [%W get]}
+	    -showvalue true -label "Volume"
+    bind .body.volume.scale <ButtonRelease-1> {VolumeCmd [%W get]}
     pack .body.volume.scale -side top -expand 1 -fill x
     frame .body.volume.mixer -relief flat -bd 0
     scale .body.volume.mixer.bass -orient horizontal -length 135\
@@ -822,7 +1026,7 @@ proc CreateWindow {} {
 	    .body.button.quit\
 	    -side left -ipadx 4 -pady 2 -fill x
 
-    if {$Config(Tracing)} {
+    if {$Stat(Tracing)} {
 	# trace display
 	TraceCreate
     }
@@ -837,6 +1041,7 @@ proc CreateWindow {} {
     bind . <Key-n> "NextCmd"
     bind . <Key-Left> "BackwardCmd"
     bind . <Key-p> "PrevCmd"
+    bind . <Key-r> "RestartCmd"
     bind . <Key-Down> "VolDownCmd"
     bind . <Key-v> "VolDownCmd"
     bind . <Key-Up> "VolUpCmd"
@@ -846,6 +1051,8 @@ proc CreateWindow {} {
     bind . <Return> "PlayCmd"
     bind . <Key-c> "StopCmd"
     bind . <Key-q> "QuitCmd"
+    bind . <Key-u> "BaseUpCmd"
+    bind . <Key-d> "BaseDownCmd"
 
     SetVolume $Config(CurVol)
 }
