@@ -13,8 +13,10 @@ extern int max_synth_voices;
 
 #define SEQUENCER_DEV	"/dev/sequencer"
 
-void seq_init(int reset);
+void seq_init(void);
+void seq_reset_samples(void);
 void seq_end(void);
+void seq_default_atten(int val);
 void seq_remove_samples(void);
 void seq_initialize_chip(void);
 void seq_set_gus_bank(int bank);
